Type: Package
Package: GeoThinneR
Title: Efficient Spatial Thinning of Species Occurrences
Version: 2.1.2
Authors@R: 
    person("Jorge", "Mestre-Tomás", , "jorge.mestre.tomas@csic.es", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-8983-3417"))
Description: Provides efficient geospatial thinning algorithms to reduce
    the density of coordinate data while maintaining spatial
    relationships. Implements K-D Tree and brute-force distance-based
    thinning, as well as grid-based and precision-based thinning methods.
    For more information on the methods, see Elseberg et al. (2012)
    <https://hdl.handle.net/10446/86202>.
License: MIT + file LICENSE
URL: https://github.com/jmestret/GeoThinneR,
        https://jmestret.github.io/GeoThinneR/
BugReports: https://github.com/jmestret/GeoThinneR/issues
Depends: R (>= 4.0.0)
Imports: data.table, doParallel, fields, foreach, graphics, grDevices,
        matrixStats, nabor, sf, stats, terra
Suggests: ggplot2, knitr, rmarkdown, s2 (>= 1.1.0), testthat (>=
        3.0.0), tibble
VignetteBuilder: knitr
BuildVignettes: true
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-25 17:20:26 UTC; jorge
Author: Jorge Mestre-Tomás [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8983-3417>)
Maintainer: Jorge Mestre-Tomás <jorge.mestre.tomas@csic.es>
Repository: CRAN
Date/Publication: 2026-08-25 22:00:14 UTC
