# Generated by roxygen2: do not edit by hand

S3method(as_sf,GeoThinned)
S3method(get_trial,GeoThinned)
S3method(largest,GeoThinned)
S3method(largest_index,GeoThinned)
S3method(plot,GeoThinned)
S3method(print,GeoThinned)
S3method(print,summary.GeoThinned)
S3method(summary,GeoThinned)
export(as_GeoThinned)
export(as_sf)
export(calculate_spatial_coverage)
export(compute_nearest_neighbor_distances)
export(compute_neighbors_brute)
export(compute_neighbors_kdtree)
export(compute_neighbors_local_kdtree)
export(distance_thinning)
export(estimate_k_max)
export(get_trial)
export(grid_thinning)
export(is_lonlat)
export(largest)
export(largest_index)
export(lon_lat_to_cartesian)
export(max_thinning_algorithm)
export(new_GeoThinned)
export(precision_thinning)
export(select_target_points)
export(thin_points)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(foreach,"%dopar%")
importFrom(graphics,legend)
importFrom(graphics,points)
